-- ------------------------------------------------------------------------- --
-- Project: ReGroup - Automatically Reassemble a Group
-- Author:  VincentSDSH				
-- Version: 1.0b - w8.0
	-- ------------------------------------------------------------------------- --
local AddonName = ...
ReGroup = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0", "AceComm-3.0", "libPortableTools-1.0")
ReGroup.wowVer = "8.0"
local debug = function(...) ReGroup:debugMsg(...) end
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName)

function ReGroup:OnInitialize()
	self:pt_Setup("ReGroup", "rg", false)

  -- Defaults -------------------------------------------------------------- --
  self.icons = {
    ["defaultLDBIcon"]  = ([[Interface\Addons\%s\icons\appIcon]]):format(AddonName),
  } 

  -- Setup AceDB
	self.DBDefaults = {
		["char"] = {
    	["DBIconTable"] = { ["hide"] = false },
		},
    ["global"] = {
    	["ldbObj_neutralText"] = L.ldbObjText_notLeader_notMember,
    	["auto_SendInvites"] = true,
    	["auto_AcceptInvites"] = true,
   		["announceInvite"] = true, 
   		["ldbIcon"] = "defaultLDBIcon",
			["max_retries"] = 5,
			["delay"] = 5,
			["QuickOpts_Scale"] = 1.2,
    },
  }
  self.db = LibStub("AceDB-3.0"):New("ReGroupDB", self.DBDefaults, true)    

	debug("* Debugging Mode Active *") -- dev sanity check
	
	-- Register Options ------------------------------------------------------ --
	self:regOptions()
	self:addBlizzardOptionsPanel()

  -- libDataBroker and DBIcon ---------------------------------------------- --
	self.AcceptInvitesFrom = L.NoLoggedGroupLeader -- GetUnitName("player", true) -- intended for humor; use wiesely and do not break causality with it.
  self.meatPopcicle = self:ldb_ObjectCreate()
  LibStub("LibDBIcon-1.0"):Register(AddonName, self.meatPopcicle, self.db.char.DBIconTable) 
end

function ReGroup:OnEnable()
	-- control vars
	self.pName = GetUnitName("player")
	self.timers = nil				 -- timer pool
	self.group = {}					 -- group members

	self:RegisterEvents()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", self.ChatEventFilter)
	
	-- Brand New User	
	if not self.db.global.IsOldHat then
		self:ScheduleTimer("OpenConfig", 5, "About")
		self.db.global.IsOldHat = true
	end

	self:RegisterComm(AddonName, "CommMsgReceived")
end

function ReGroup:RegisterEvents()
	local globalOpts = self.db.global
	
	if globalOpts.auto_SendInvites then 
		self:RegisterEvent('GROUP_ROSTER_UPDATE',  "RosterUpdate")
	else
		self:UnregisterEvent('GROUP_ROSTER_UPDATE',  "RosterUpdate")
	end	
	
	if globalOpts.auto_AcceptInvites then 
		self:RegisterEvent('PARTY_INVITE_REQUEST', "InviteRequest")
	else
		self:UnregisterEvent('PARTY_INVITE_REQUEST', "InviteRequest")	
	end		
end

function ReGroup:CommMsgReceived(prefix, message, distribution, sender)
	if sender==self.pName then return end -- since we have no self-reactives based on comm traffic, no need to process our own messages, 
																				-- even though most enlightened people see nothing wrong with talking to yourself now and then.	
	if message==L.DelistMe then
		self:alert("Delisting Request Received: |cffff9966"..sender.."|r")
		ReGroup:DelistMember(sender)
	end
end

-- libDataBroker handling funcs
function ReGroup:heuristicAnchor(frame, obj)
	if frame and obj then
		local frame_pt, obj_pt
		local x, y = obj:GetCenter()

		if not x or not y then 
			frame_pt ="TOP"; obj_pt = "BOTTOM"
		else
			frame_pt = (y > UIParent:GetHeight() / 2) and "TOP" or "BOTTOM"
			obj_pt = frame_pt == "TOP" and "BOTTOM" or "TOP"
		end

		frame:SetClampedToScreen(true)
		frame:ClearAllPoints()
		frame:SetPoint(frame_pt, obj, obj_pt)	
	else
		if obj then ReGroup:alert("ERROR: frame=nil in :heuristicAnchor(frame, obj)")
		else        ReGroup:alert("ERROR: obj=nil in :heuristicAnchor(frame, obj)")
		end
	end
end

function ReGroup:getText_ldbObject() 
	local text

	if self.isGroupLeader or self.isGroupMember then -- may have both roles
		if self.isGroupMember then
			if self.AcceptInvitesFrom then text = "L: |cffff9966"..self.AcceptInvitesFrom.."|r" end	
		end
		
		if self.isGroupLeader then
			if text then text=text..": " 
			else text = "" 
			end
		
			text = text.. (self.group and tostring(#self.group) or "0").." - "..(self.db.global.auto_SendInvites and "A" or "M") 
		end
	
	else -- neutral
		text = self.db.global.ldbObj_neutralText or L.ldbObjText_notLeader_notMember
	end

	return text
end
function ReGroup:update_ldbObject() self.meatPopcicle.text = self:getText_ldbObject() end
function ReGroup:ldb_ObjectCreate()
	local globalOpts = self.db.global
	
	return LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
		type = "launcher",
		label = L.appName, 
		text = self:getText_ldbObject(),
		icon = self.icons[ReGroup.db.global.ldbIcon],
		OnTooltipShow = function(tooltip)
			tooltip:AddLine( self:ColorPencil( " - ", "seagreen", "orange").. 
				L.ldbTooltip_title ..
				self:Crayon("seagreen").." - "
			)
			tooltip:AddLine(" ")
			
			-- Accepting From
			if globalOpts.auto_AcceptInvites then			
				tooltip:AddLine(self:Crayon("brightgrass")..L.ldbTooltip_AutoAccept:format(self:Crayon("unorange")..self.AcceptInvitesFrom) )
			else
				tooltip:AddLine(
					self:Crayon("brightgrass")..L.ldbTooltip_AutoInv_pt1 .. self:Crayon("crimson")..L.ldbTooltip_AutoInv_pt2
				)
			end
			tooltip:AddLine(" ")
		
			-- Sending To	
			if self.group and #self.group>0 then
				tooltip:AddLine( self:Crayon("orange")..L.ldbTooltip_AutoInviteMembers ) 
				for k, v in pairs(self.group) do
					tooltip:AddLine(self:Crayon("unorange")..v)
				end
			else
				tooltip:AddLine(self:Crayon("unorange")..L.ldbTooltip_NoLoggedMembers)
			end

		end,			
		OnClick = function(ldbObj, button)
			if IsShiftKeyDown() then
				if button == "RightButton" then 
					self:leaveGroup()
				else
					if not IsInGroup() then
						self:alert("Stop playing around: |cff6699ffYOU ARE NOT IN A GROUP|r")
					elseif UnitIsGroupLeader("player") then
						self:set_LeaderData()
					else
						self:set_MemberData()
					end					
				end
			else
				if button == "RightButton" then	
					self:quickOptions(ldbObj)
				else
					if self.group and #self.group>0 then
						self:KillTimers()
						self:RosterUpdate("OnClick")
					else
						self:alert("No Group Members to Invite")
					end
				end
			end
		
		end,
	})
end

