-- ------------------------------------------------------------------------- --
-- Project: ReGroup - Automatically Reassemble a Group
-- 					Configuration
-- Author:  VincentSDSH				
-- Version: 1.0 - w7.2
-- ------------------------------------------------------------------------- --
local AddonName = ...
ReGroup.Dialog = LibStub("AceConfigDialog-3.0")
local debug = function(...) ReGroup:debugMsg(...) end
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName)

-- simple color funcs
local Crayons = {
  -- whites
 	["cream"] = "deb887",
  -- purples
 	["lilac"]  = "d1c4e7",
 	["purple"] = "8b87de",
  -- greens
 	["grass"] = "aede87",
	-- blues
 	["sky"]  = "87afde",
 	["cyan"] = "87d8de",

	["white"] = "FFFFFF",
	["black"] = "000000",

	["lightgray"] = "cccccc",
	["fuglyYellow"] = "FEDF0B",
	["dullRed"] = "aa0000",
	
	["hard_red"] = "ff0000",
	["hard_blue"] = "0000ff",
	["hard_green"] = "00ff00",
 
 	["crimson"] = "DC143C",
 	
 	["orange"] = "ff9966",
 	["redvelvet"] = "ff6699",
	["unorange"] = "6699ff", 
	["violet"] = "9966ff",
	["seagreen"] = "66ff99",
	["brightgrass"] = "99ff66",
	
	["sunshine"] = "ffff00",
	["afternoon"] = "dddd00",   

	["richteal"] = "00ffff",
	["richblue"] = "0099ff",
	["richgreen"] = "00ff99",
	["easygray"] = "888888",
	["easygreen"] = "00aa00",	

}
function ReGroup:Crayon(color) 
	local colorCode = Crayons[color]
	return colorCode and ("|cff"..colorCode) or ("[color: "..color.." not found]")
end
function ReGroup:ColorPencil(value, sColor, eColor)
  sColor = sColor and self:Crayon(sColor) or ""
  eColor = sColor and ( eColor and self:Crayon(eColor) or "|r" ) or ""
	return sColor..value..eColor	
end

-- Blizzy Opts
function ReGroup:addBlizzardOptionsPanel()   
	local charOpts = self.db.char
	local blizzy_Opts = {		
		type = "group",
		order = 10,
		name = ("\124T%s:%d:%d:1:0\124t %s"):format(self.icons.defaultLDBIcon, 32, 32, AddonName),
		args = {
			introspacer = { order = 4, type = "description", name = "  \n"}, 
			intro = {
				order = 5,
				type = "description",
				fontSize = "large",
				name = L.addonShortDescription.."\n\n",
			},
			configure = {
			  type = "execute",
				name = "Configure ReGroup",
			  order = 50,
				func = function()
					-- IOFC must come before GMBC b/c GameMenu is hidden when IOF is shown, when closed, the Game Menu re-openes 
					-- if IOF opened from there; calling GMBC:Click() has no harmful effects regardless of Game Menu status.
					GameTooltip:Hide(); InterfaceOptionsFrameCancel:Click(); GameMenuButtonContinue:Click()
					self:OpenConfig("About")
				end, 
			},					
			verionData = {
				order = 100,
				type = "description",
				fontSize = "medium",
				name = ("\n%sAddon version %s updated to WoW version %s by %s"):format(
							self:Crayon("cream"),
							self:ColorPencil(GetAddOnMetadata(AddonName, 'Version'), "grass", "cream"), 
							self:ColorPencil(L.wowVer, "redvelvet", "cream"), 
							self:ColorPencil(L.authorName, "orange")
							),
			},
		},
	}
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(AddonName, blizzy_Opts)	-- Blizzy Opts -
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName, AddonName) 	      ----------------	
end

-- Main Opts
function ReGroup:OpenConfig(tab)
	local opts = self.opts
	self.Dialog:Open(opts.f_reg)
	
	local openTab = opts["f_"..tostring(tab)]
	self.Dialog:SelectGroup(opts.f_reg, true and openTab or opts.f_MainOpts)
end

function ReGroup:makeCommonHeader(hdrName, numOrder, label)
		return {
							order = true and numOrder or 0, type = "description", fontSize = "large",
							name = function()
								-- Status Field Update Cheat: alters the Status Field when the tab is changed
								self.Dialog.OpenFrames[self.opts.f_reg]:SetStatusText(
									("%s (v%s) by %s  %s: %s"):format( 
											self:ColorPencil(AddonName, "seagreen", "cream"),
											self:ColorPencil(GetAddOnMetadata(AddonName, 'Version'), "grass", "cream"), 
											self:ColorPencil(L.authorName, "orange"),
											self:ColorPencil("Page", "cyan", "white"),
											self:ColorPencil(label or hdrName, "grass")
									) 
								) 								
								return "|cffffff00"..hdrName.."|r\n"
							end
					 }
end

function ReGroup:regOptions()
	local function mkLink(path, size) return ("\124T"..path..":%d:%d:1:0\124t"):format(size, size) end
	local charOpts, globalOpts = self.db.char, self.db.global
	-- Options Tables 
	amalgam = { 
		type = "group", 
		childGroups = "tab",
	  name = AddonName, 
		args = {
			About = {
				type = "group",
				order = 10,
				name = mkLink(self.icons.defaultLDBIcon, 4).." ".. L.opt_tab_About .." "..mkLink(self.icons.defaultLDBIcon, 4),
				args = {
					header = self:makeCommonHeader(mkLink(self.icons.defaultLDBIcon, 32).."  "..AddonName..": "..L.opt_tab_About, 0, L.opt_tab_About),
					intro = {
						order = 1,
						type = "description",
						fontSize = "large",
						name = L.addonShortDescription,
					},
					introspacer = { order = 2, type = "description", name = "  "}, 
					versionData = {
						order = 3,
						type = "description",
						fontSize = "medium",
						name = L.versionData:format(
							self:Crayon("cream"),
							self:ColorPencil(GetAddOnMetadata(AddonName, 'Version'), "grass", "cream"), 
							self:ColorPencil(L.wowVer, "redvelvet", "cream"), 
							self:ColorPencil(L.authorName, "orange")
							),
					},
						
					ldbAnchors = {
				    type = "group", 
				    inline = true, 
				    name = " ", 
				    order = 10,
						args = {
							hideMinimapIcon = {
								order = 20, 
								type = "toggle", 
								name = L.opt_HideMinimapIcon, 
								get = function(info) return charOpts.DBIconTable.hide end,
								set = function(info, value)
									charOpts.DBIconTable.hide = value
									if value then
										LibStub("LibDBIcon-1.0"):Hide(AddonName) 
									else 
										LibStub("LibDBIcon-1.0"):Show(AddonName) 
									end
								end,
							},				
						},
					},
				},
			},
			Help = {
				type = "group",
				order = 10,
				name = mkLink(self.icons.defaultLDBIcon, 4).." ".. L.opt_tab_Help .." "..mkLink(self.icons.defaultLDBIcon, 4),
				args = {
					header = self:makeCommonHeader(mkLink(self.icons.defaultLDBIcon, 32).."  "..AddonName..": "..L.opt_tab_Help, 0, L.opt_tab_Help),
					intro = {
						order = 1,
						type = "description",
						fontSize = "large",
						name = L.Help,
					},
				},
			},
			mainOpts = {
				type = "group",
				order = 50,
				name = mkLink(self.icons.defaultLDBIcon, 4).."  ".. L.opt_tab_MainOpts .." "..mkLink(self.icons.defaultLDBIcon, 4),
				args = {
					header = self:makeCommonHeader(mkLink(self.icons.defaultLDBIcon, 32).."  "..AddonName..": "..L.opt_tab_MainOpts, 0, L.opt_tab_MainOpts),
					Anncouncements = {
				    type = "group", 
				    inline = true, 
				    name = L.opt_Anncouncements, 
				    order = 10,			
						args = {
							intro = {
								order = 1,
								type = "description",
								fontSize = "large",
								name = L.opt_Anncouncements_intro,
							},
							announceInvite = {
								order = 10, 
								type = "toggle", 
								name = L.opt_Anncouncements_announceInvite, 
								width = "double",
								get = function(info) return globalOpts.announceInvite end,
								set = function(info, value) globalOpts.announceInvite = value end,
							},							
						},
					},
					Regrouping = {
				    type = "group", 
				    inline = true, 
				    name = L.opt_Regrouping, 
				    order = 20,			
						args = {
							hdr1 = {type="header", order=5, name=L.opt_Regrouping_hdr1},
							auto_AcceptInvites = {
								type = "toggle",
								order = 20,
								name = L.opt_Regrouping_auto_AcceptInvites_name,
								desc = L.opt_Regrouping_auto_AcceptInvites_desc,
								width = "full",
								get = function(info) return globalOpts.auto_AcceptInvites end,
								set = function(info, value) 
									globalOpts.auto_AcceptInvites = value
									if not value and self.isGroupMember then self:clear_MemberData() end
									self:RegisterEvents()
									self:update_ldbObject()
								end,
							},							
							spacer00 = {order = 28, type = "description", name = " "},
							hdr2 = {type="header", order=29, name=L.opt_Regrouping_hdr2},
							auto_SendInvites = {
								type = "toggle",
								order = 30,
								name = L.opt_Regrouping_auto_SendInvites_name,
								desc = L.opt_Regrouping_auto_SendInvites_desc,
								width = "full",
								get = function(info) return globalOpts.auto_SendInvites end,
								set = function(info, value) 
									globalOpts.auto_SendInvites = value
									self:RegisterEvents()
									self:update_ldbObject() 
								end,
							},
							spacer01 = {order = 31, type = "description", name = " "},
							intro = {
								order = 40,
								type = "description",
								disabled = function() return not globalOpts.auto_SendInvites end,
								fontSize = "large",
								name = L.opt_Regrouping_intro,
							},
							max_retries = {
							  type = "range",
							  disabled = function() return not globalOpts.auto_SendInvites end,
							  name = L.opt_Regrouping_max_retries,
							  order = 50,
							  step = 1, min = 1, max = 60,
							  isPercent = false,
								get = function() return globalOpts.max_retries end,
								set = function(i, v) globalOpts.max_retries = v end,
							}, 
							delay = {
							  type = "range",
							  disabled = function() return not globalOpts.auto_SendInvites end,
							  name = L.opt_Regrouping_delay,
							  order = 60,
							  step = 1, min = 1, max = 60,
							  isPercent = false,
								get = function() return globalOpts.delay end,
								set = function(i, v) globalOpts.delay = v end,
							}, 							
						},
					},
					ldbObj = {
						type = "group",
						inline = true,
						name = L.opt_ldbObj,
						order = 30,
						args = {
							QuickOpts_Scale = {
								type = "range",
							  name = L.opt_QuickOpts_Scale,
							  order = 10,
							  step = .05, min = .5, max = 3,
							  isPercent = false,
								get = function() return globalOpts.QuickOpts_Scale end,
								set = function(i, v) globalOpts.QuickOpts_Scale = v end,								
							},
							neutralText = {
							  type = "input",
							  name = L.opt_ldbObj_neutralText_name,
							  desc = L.opt_ldbObj_neutralText_desc,
							  width = full,
							  get = function()
							    return globalOpts.ldbObj_neutralText
							  end,
							  set = function(info, value)
							    globalOpts.ldbObj_neutralText = tostring(value)
							    self:update_ldbObject()
							  end,
							  width="full",
							  order = 20
							},						
						},
						
					},
				},
			},
		}			
	}

	-- Register Options Tables
	self.opts = {f_reg = "ReGroupConfig", f_About = "About", f_Help = "Help", f_MainOpts = "mainOpts"}
 	local opts = self.opts
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable( opts.f_reg, amalgam ) 
	self.Dialog:SetDefaultSize( opts.f_reg, 800, 700 ) 	
end 

-- Quick Opts
function ReGroup:toggle_auto_SendInvites()
	local globalOpts = self.db.global
	globalOpts.auto_SendInvites = not globalOpts.auto_SendInvites
	self:RegisterEvents()
	self:update_ldbObject()
end

function ReGroup:quickOptions(ldbObj)
	GameTooltip:Hide()

	local optTable = {
	  type='group',
	  order=1,
	  args = {
			About = {
	      type = "execute",
	      name = L.opt_tab_About,
	      func = function() self:OpenConfig("About") end,
	      order = 10		
			},
			Help = {
	      type = "execute",
	      name = L.opt_tab_Help,
	      func = function() self:OpenConfig("Help") end,
	      order = 20		
			},
			Options = {
	      type = "execute",
	      name = L.opt_tab_Opts,
	      func = function() self:OpenConfig() end,
	      order = 30		
			},			

	    hdr3 = {
	      type = "header",
	      name = L.optTable_hdr3,
	      dropdownHidden = (not self.isGroupLeader) and (not self.isGroupMember),
	      order = 90
	    }, 
			clearAll = {
	      type = "execute",
	      name = L.optTable_clearAll,
	      desc = "",
	      dropdownHidden = (not self.isGroupLeader) and (not self.isGroupMember), 
	      func = function() self:clear_AllData() end,
	      order = 100	
			},
			clearLeader = {
	      type = "execute",
	      name = L.optTable_clearMemberList,
	      dropdownHidden = not self.isGroupLeader, 
	      func = function() self:clear_LeaderData() end,
	      order = 110	
			},
			clearMember = {
	      type = "execute",
	      name = L.optTable_clearLeaderName,
	      dropdownHidden = not self.isGroupMember, 
	      func = function() self:clear_MemberData() end,
	      order = 120	
			},
								
	    hdr5 = {
	      type = "header",
	      name = L.optTable_hdr5,
	      dropdownHidden = not IsInGroup(),
	      order = 200
	    }, 	
			setLeader = {
	      type = "execute",
	      name = L.optTable_setMemberList,
	      dropdownHidden = not IsInGroup(),
	      func = function() self:set_LeaderData() end,
	      order = 210	
			},
			setMember = {
	      type = "execute",
	      name = L.optTable_setLeaderName,
	      dropdownHidden = not IsInGroup() or UnitIsGroupLeader("player") or not self.db.global.auto_AcceptInvites,
	      func = function() self:set_MemberData() end,
	      order = 220	
			},						
	
			hdr6 = {
	      type = "header",
	      name = L.optTable_hdr6,
	      dropdownHidden = not self.isGroupLeader,
	      order = 300
	    }, 	
			toggle_auto_SendInvites = {
	      type = "execute",
	      name = L.optTable_toggle_auto_SendInvites,
	      dropdownHidden = not self.isGroupLeader,
	      desc = "",
	      func = function() self:toggle_auto_SendInvites() end,
	      order = 310	
			},

	    hdr10 = {
	      type = "header",
	      name = L.optTable_hdr10,
	      dropdownHidden = not IsInGroup(),
	      order = 800
	    }, 	
	    LeaveGroup = {
	      type = "execute",
	      name = L.optTable_LeaveGroup,
				dropdownHidden = not IsInGroup(),
	      func = function() self:leaveGroup() end,
	      order = 810
	    },	
	  }
	}		
				
	local menu = LibStub("LibDropdown-1.0"):OpenAce3Menu(optTable)
	-- menu:SetPoint("BOTTOMLEFT", ldbObj, "TOPLEFT")
	self:heuristicAnchor(menu, ldbObj)
	menu:SetScale(self.db.global.QuickOpts_Scale)
	
	menu:SetScript("OnLeave", function(self) if not self:IsMouseOver() then self:Hide() end end)
end