-- ------------------------------------------------------------------------- --
-- Project: ReGroup - Automatically Reassemble a Group
-- 					Core Functions
-- Author:  VincentSDSH				
-- Version: 1.0 - w7.2
-- ------------------------------------------------------------------------- --
local AddonName = ...
local debug = function(...) ReGroup:debugMsg(...) end
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName)

function ReGroup:clear_AllData()
	self:clear_LeaderData()
	self:clear_MemberData()
end
function ReGroup:clear_LeaderData()
	self.isGroupLeader = nil
	self:KillTimers()
	self.group = {}
	self:alert("Leader Data Cleared")
	self:update_ldbObject()	
end
function ReGroup:clear_MemberData()
	self.isGroupMember = nil
	self.AcceptInvitesFrom = L.NoLoggedGroupLeader
	self:alert("Member Data Cleared")
	self:update_ldbObject()	
end 	
 
function ReGroup:set_LeaderData()
	self.isGroupLeader = true -- simplifies ldbObj.text building
	
	-- Gather Party Names
	self.group = {}
	for i=1,4 do
		if UnitExists("party"..tostring(i)) then
			tinsert(self.group, GetUnitName("party"..tostring(i), true))
		end
	end							
	self:update_ldbObject()
	self:alert("Party Gathered: |cff6699ff"..tostring(#self.group).." members|r")
	
	-- automatically set invite mode to MANUAL if user is not GroupLeader
	if not UnitIsGroupLeader("player") then self.db.global.auto_SendInvites = false; self:update_ldbObject() end
end
function ReGroup:set_MemberData()
	if UnitIsGroupLeader("player") then
		self:alert("Stop playing around: |cff6699ffYOU CANNOT ACCEPT GROUP INVITES FROM YOURSELF|r")
	else
		self.isGroupMember = true -- simplifies ldbObj.text building
		
		-- Accept Invites from Party Leader
		for i=1,4 do
			local p = "party"..tostring(i)
			if UnitExists(p) and UnitIsGroupLeader(p) then
				self.AcceptInvitesFrom = GetUnitName("party"..tostring(i), true)
				break;
			end
		end
		self:update_ldbObject()
		self:alert("Now Accepting Invites From: |cff6699ff"..self.AcceptInvitesFrom.."|r")
	end
end

function ReGroup:DelistMember(gpMember) -- removes "gpMember" from the self.group list
		if ReGroup.group and gpMember then
			for k, v in pairs(ReGroup.group) do
				if v == gpMember then
					table.remove(ReGroup.group, k)
					ReGroup:update_ldbObject()
					break;
				end
			end
		end

end

function ReGroup:GetParty() -- returns current state of the party
	local gp = {}
	for i=1,4 do
		local p = "party"..tostring(i)
		if UnitExists(p) then
			gp[ GetUnitName("party"..tostring(i), true) ] = true
		end
	end
	return gp
end

function ReGroup:KillTimers()
	self:CancelAllTimers()
	self.timers = nil
end

function ReGroup:doPartyInvites(retry)
	local announceInvite = self.db.global.announceInvite
	local gp = self:GetParty()	
	local invitesSent = 0
	for k, v in pairs(self.group) do
		if not gp[ v ] then
			InviteUnit(v)
			if announceInvite then self:alert("Inviting "..v) end
			invitesSent = invitesSent + 1
		end
	end

	-- (invitesSent == 0) = all group members added
	if (invitesSent == 0) or (retry == self.db.global.max_retries) then
		self:KillTimers()
	end

end
	-- Event Handlers 
function ReGroup:RosterUpdate(event)
	-- fires w/ invites and at party exit, can trigger timer to 'reinvite'
	-- Choosing to spawn max_retries of timers rather than schedule a repeating timer and track iterations
	local globalOpts = self.db.global
	if not IsInGroup() then -- Only automatically trigger invite storm if GroupLeader is NOT in a group.
		if not self.timers and (self.group and #self.group>0) then -- spawn timers if no timers
			self:doPartyInvites(0) -- spawn one round immediately, since the other Group Members may have already left the temp group
			
			self.timers = {}
			for i=1,globalOpts.max_retries do
				self.timers["ReGroup_Attempt"..tostring(i)] = self:ScheduleTimer("doPartyInvites", (globalOpts.delay * i), i )			
			end
		end
	end
end	

function ReGroup:ChatEventFilter(event, msg)
	-- "Player-Server Name declines your group invitation."
	if msg:match(L.inviteDeclined_Seek) then
		local inviteeName = msg:match(L.inviteDeclined_Mask)
		if inviteeName then ReGroup:DelistMember(inviteeName) end
	end
end

function ReGroup:InviteRequest(event, partyLeader)
	if self.AcceptInvitesFrom == partyLeader then
		self:inviteAccept(partyLeader)
	-- else
		-- No criteria established for this, normal invite window should show
		-- self:inviteDecline(partyLeader)		
	end
end	

	-- Invite Handlers
function ReGroup:inviteAccept(partyLeader)
	self:alert("Accepting Invite from |cff6699ff"..partyLeader.."|r")
	AcceptGroup()
	for i = 1, STATICPOPUP_NUMDIALOGS do
		local StaticPopup = _G["StaticPopup" .. i]
		if StaticPopup.which == "PARTY_INVITE" then
			StaticPopup.inviteAccepted = 1
			break
		end
	end	
	StaticPopup_Hide("PARTY_INVITE")
end

function ReGroup:inviteDecline(partyLeader)
	self:alert("Rejecting Invite from |cff6699ff"..partyLeader.."|r")
	DeclineGroup()
	StaticPopup_Hide("PARTY_INVITE")	
end

 -- Other
function ReGroup:leaveGroup()
	self.AcceptInvitesFrom = L.NoLoggedGroupLeader
	self:SendCommMessage(AddonName, L.DelistMe, "PARTY")
	self:alert("Delisting Request Sent")
	self:ScheduleTimer(function() LeaveParty() end, 1)
end