-- ---------------------------------------------------------------------------------- --
-- 	libPortableTools         																													--
-- 	Ace Embeddable Tools Lib 
-- 	vincentSDSH																																				--
-- ---------------------------------------------------------------------------------- --

local PT_MAJOR, PT_MINOR = "libPortableTools-1.0", 3
local lib, oldminor = LibStub:NewLibrary(PT_MAJOR, PT_MINOR)

if not lib then return end -- No upgrade needed


--[ housekeeping ] ------------------------------------------------- --
local Crayons = {
	["PTtan"] = "ff9966",  -- orange
	["PTblue"] = "6699ff", -- unoraange
}
local function cw(color, msg)
	return "|cff"..color..(true and msg or "nil").."|r"
end

--[ setup tags ] --------------------------------------------------- --
function lib:pt_Setup(name_alert, name_debug, enableDebug) -- single call setup
	self.libPT_name_alert = name_alert
	self.libPT_name_debug = name_debug
	self.libPT_enableDebug = enableDebug
end

function lib:pt_enableDebug(enableDebug) -- allows toggle on/off
	self.libPT_enableDebug = enableDebug
end

--[ debug function ] ----------------------------------------------- --
-- :debugMsg(data[[, label], colorize])
-- Output: 
--			debugMsg(data) -> [data]
--			debugMsg(data, label) -> [label] is [data]
--			debugMsg(data, label, true) -> data treated as boolean, '[label] is [data]' is colored Red=nil/false, Green=true
function lib:debugMsg(data, label, colorize)
	if self.libPT_enableDebug then 
		local r, b, g 
		if not colorize then r=1; g=1; b=.8 
		elseif (label and data) then r=0; g=1; b=0 
		else r=1; g=0; b=0 
		end; 
		
		local bigColon = cw(Crayons["PTblue"], "::")
		
		DEFAULT_CHAT_FRAME:AddMessage( bigColon..cw(Crayons["PTtan"], self.libPT_name_debug)..bigColon .." ".. (label and (label.." is: ") or "").. tostring(data), r, g, b); 
	end
end

--[ addon Alert msg ] ---------------------------------------------- --	
function lib:alert(msg, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage( cw(Crayons["PTtan"], self.libPT_name_alert)..cw(Crayons["PTblue"],":")..tostring(msg), r or 1, g or 1, b or .8);
end

-- [ Embeds ] ----------------------------------------------------------- --
lib.embeds = lib.embeds or {} -- table containing objs lib is embedded in

local mixins = { -- functions to expose to addon
	"pt_Setup",
	"pt_enableDebug",
	"debugMsg", 
	"alert"
}
function lib:Embed( target )
	for k, v in pairs( mixins ) do
		target[v] = self[v]
	end
	self.embeds[target] = true
	return target
end

function lib:OnEmbedEnable( target ) 
	-- defaults, just in case the dev forgets to prime
	self.libPT_name_alert = "Portable Tools"
	self.libPT_name_debug = "pt"
end
function lib:OnEmbedDisable( target ) end
for addon in pairs(lib.embeds) do lib:Embed(addon) end

--[[ BoneYard ------------------
local function checkSelf()
	for k, v in pairs(self) do
		if k:sub(1,3) == "lib" then
		print(type(k).." "..k)
		end
	end
end
--]] ---------------------------